/* Project name:
  1   Matrix Code
 * Copyright:
     (c) MikroElektronika, 2011.
 * Revision History:
     20101224:
       - initial release (AN);
     20110210:
       - added header comments (SZ);
 * Description:
       Matrix code animation at TFT 320x240 display.
 * Test configuration:
     MCU:             dsPIC33FJ128GP710
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70286C.pdf
     Dev.Board:       mikroMMB for dsPIC33 Board
                      http://www.mikroe.com/eng/products/view/586/mikrommb-for-dspic33-board/
     Oscillator:      HS-PLL, 64.00000 MHz
                      ac:TFT_display
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/eng/products/view/230/mikropascal-pro-for-dspic30-33-and-pic24/
 * NOTES:
     - none.
 */

#include "matrix_driver.h"
TMatrixCode myMatrixCode;

void setInitValues(){
  AD1PCFGL = 0xFFFF;
  AD1PCFGH = 0xFFFF;
  AD2PCFGL = 0xFFFF;
  
  // Set PLL : Fosc = ((Fin/PLLPRE)*PLLDIV)/PLLPOST ; (((8MHz/2)*32)/2) = 64MHz
  // refer to the family datasheet for more details
  CLKDIVbits.PLLPRE = 0;
  PLLFBDbits.PLLDIV = 0x1E;
  CLKDIVbits.PLLPOST = 0;

   // Disable other peripheral modules on the same SPI bus
  TRISC4_bit = 0;
  LATC4_bit = 1;   // Disable Serial Flash module
  Delay_ms(150);
  InitializeTFT();
  initMatrixCode(&myMatrixCode, 13, 17);
}

void main() {
  setInitValues();
  while(1){
    DoMatrix(&myMatrixCode);
  }
}