#include <built_in.h>
#include "matrix_fonts.h"
#include "matrix_defs.h"

// TFT module connections
unsigned short TFT_DataPort at LATD;                         // DATA port
unsigned short TFT_DataPort_Direction at TRISD;              // DATA port

sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_RST  at LATC1_bit;

sbit TFT_RS_Direction  at TRISB15_bit;
sbit TFT_RS   at LATB15_bit;

sbit TFT_CS_Direction  at TRISC3_bit;
sbit TFT_CS   at LATC3_bit;

sbit TFT_RD_Direction  at TRISD12_bit;
sbit TFT_RD   at LATD12_bit;

sbit TFT_WR_Direction  at TRISD13_bit;
sbit TFT_WR   at LATD13_bit;

static void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Lo(LATD) = index;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write command
static void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Lo(LATD) = cmd;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write data
static void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Lo(LATE) = Hi(_data);
  Lo(LATD) = Lo(_data);
  TFT_WR = 0;
  TFT_WR = 1;
}

void InitializeTFT() {
  TRISE = 0;
  TRISD = 0;
  Delay_ms(150);
  TFT_Set_Active(Set_Index,Write_Command,Write_Data);
  TFT_Init(240, 320);
  TFT_Fill_Screen(CL_BLACK);
  TFT_Set_Pen(CL_BLACK, 0);
}

void initMatrixCode(TMatrixCode * theMatrixCode, unsigned short NumColumns, unsigned short NumCharsInColumn){
  unsigned int i, j;
  TMatrixColumn *CurrentColumn;
  if (theMatrixCode > 0){
    theMatrixCode->Font = Matrix_Code_NFI9x19;
    theMatrixCode->FontHeight = 19;
    theMatrixCode->FontWidth = 19;
    theMatrixCode->MaxCharsInColumn = NumCharsInColumn;
    theMatrixCode->MaxColumns = NumColumns;
    theMatrixCode->ColorDecayDecrement = (255 / NumCharsInColumn)+1;
    srand(65);
    for (i = 0; i < NumColumns; i++){
      CurrentColumn = &theMatrixCode->EntireMatrix[i];
      for (j = 0; j < NumCharsInColumn; j++){
        CurrentColumn->ColumnChars[j]->theChar = 0;
        CurrentColumn->ColumnChars[j]->ColorIntensity = 255;
      }
      CurrentColumn->Length = rand() % NumCharsInColumn;
      for (j = 0; j < CurrentColumn->Length; j++)
        CurrentColumn->ColumnChars[j]->theChar = (rand() % 95) + 32;
    }
  }
}

static void CalculateNewMatrix(TMatrixCode * theMatrixCode){
  unsigned int i, j;
  unsigned short RowLength;
  TMatrixChar *CurrentChar;
  TMatrixColumn *CurrentColumn;
  
  for (i = 0; i < theMatrixCode->MaxColumns; i++){
    CurrentColumn = &theMatrixCode->EntireMatrix[i];
    for (j = 0; j < theMatrixCode->MaxCharsInColumn; j++){
      if (CurrentColumn->ColumnChars[j]->ColorIntensity > theMatrixCode->ColorDecayDecrement)
        CurrentColumn->ColumnChars[j]->ColorIntensity -= theMatrixCode->ColorDecayDecrement;
      else
        CurrentColumn->ColumnChars[j]->ColorIntensity = 0;
    }
    if (CurrentColumn->Length < theMatrixCode->MaxCharsInColumn)
      CurrentColumn->Length++;
    else
      CurrentColumn->Length = 1;

    CurrentColumn->ColumnChars[CurrentColumn->Length-1]->theChar = (rand() % 95) + 32;
    CurrentColumn->ColumnChars[CurrentColumn->Length-1]->ColorIntensity = 255;
  }
}

void DoMatrix(TMatrixCode * theMatrixCode){
  unsigned int i, j;
  unsigned int CharColor;
  TMatrixChar *CurrentChar;
  CalculateNewMatrix(theMatrixCode);
  for (i = 0; i < theMatrixCode->MaxCharsInColumn; i++){
    TFT_Set_Brush(1, CL_BLACK, 0, 0, 0, 0);
    for (j = 0; j < theMatrixCode->MaxColumns; j++){
      CurrentChar = &theMatrixCode->EntireMatrix[j]->ColumnChars[i];
      CharColor = TFT_RGBToColor16bit(0, CurrentChar->ColorIntensity, 0);
      if (CurrentChar->theChar >= 32) {
        TFT_Set_Brush(1, CL_BLACK, 0, 0, 0, 0);
        TFT_Rectangle(j*19, i*19+1, (j+1)*19, (i+1)*19+1);
        TFT_Set_Font(Matrix_Code_NFI9x19, CharColor, 0);
        TFT_Write_Char(CurrentChar->theChar, j*19, i*19);
      }
    }
  }
}