#include "simple_maze_objects.h"
#include "simple_maze_resources.h"
#include "built_in.h"

// TFT module connections
char TFT_DataPort at LATD;
char TFT_DataPort_Direction at TRISD;
sbit TFT_RST at LATC1_bit;
sbit TFT_BLED at LATC2_bit;
sbit TFT_RS at LATB15_bit;
sbit TFT_CS at LATC3_bit;
sbit TFT_RD at LATD12_bit;
sbit TFT_WR at LATD13_bit;
sbit TFT_RST_Direction at TRISC1_bit;
sbit TFT_BLED_Direction at TRISC2_bit;
sbit TFT_RS_Direction at TRISB15_bit;
sbit TFT_CS_Direction at TRISC3_bit;
sbit TFT_RD_Direction at TRISD12_bit;
sbit TFT_WR_Direction at TRISD13_bit;
// End Glcd module connections

// Touch Panel module connections
sbit DriveX_Left at LATB13_bit;
sbit DriveX_Right at LATB11_bit;
sbit DriveY_Up at LATB12_bit;
sbit DriveY_Down at LATB10_bit;
sbit DriveX_Left_Direction at TRISB13_bit;
sbit DriveX_Right_Direction at TRISB11_bit;
sbit DriveY_Up_Direction at TRISB12_bit;
sbit DriveY_Down_Direction at TRISB10_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
char PenDown;
void *PressedObject;
int PressedObjectType;
unsigned int caption_length, caption_height;
unsigned int display_width, display_height;

// set index
void Set_Index(unsigned short index) {
  TFT_RS = 0;
  Lo(LATD) = index;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write command
void Write_Command(unsigned short cmd) {
  TFT_RS = 1;
  Lo(LATD) = cmd;
  TFT_WR = 0;
  TFT_WR = 1;
}

// write data
void Write_Data(unsigned int _data) {
  TFT_RS = 1;
  Lo(LATE) = Hi(_data);
  Lo(LATD) = Lo(_data);
  TFT_WR = 0;
  TFT_WR = 1;
}

static void InitializeTouchPanel() {
  AD1PCFGL = 0xCFFF;
  AD1PCFGH = 0xCFFF;
  TRISE = 0;
  TRISD = 0;
  CLKDIVbits.PLLPRE = 0;
  PLLFBD = 0x30;
  CLKDIVbits.PLLPOST = 1;
  Delay_ms(150);
  ADC1_Init();
  TFT_Set_Active(Set_Index,Write_Command,Write_Data);
  TFT_Init(320, 240);

  TP_TFT_Init(320, 240, 13, 12);                                  // Initialize touch panel
  TP_TFT_Set_ADC_Threshold(900);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
  PressedObjectType = -1;
}


/////////////////////////
  TScreen *CurrentScreen;

  TScreen                Screen1;
  TLine                  Line1;
  TLine                  Line2;
  TCircle                Circle1;
  TImage                 Image3;
  TImage                 Image1;
  TImage                 Image2;
  TImage                 Image4;
  TLine                  Line3;
  TImage                 Image5;
  TImage                 *Screen0_Images[5];
  TCircle                *Screen0_Circles[1];
  TLine                  *Screen0_Lines[3];

  TScreen                Screen2;
  TLabel                 Label1;
  char Label1_Caption[17] = "Simple Maze Demo";

  TLabel                 Label2;
  char Label2_Caption[55] = "This demo shows, how simple you can make TFT graphics.";

  TLabel                 Label3;
  char Label3_Caption[60] = "The point is to guide the ball through this simple maze, by";

  TLabel                 Label4;
  char Label4_Caption[61] = "using arrows located in the right bottom part of the screen.";

  TButton                Button1;
  char Button1_Caption[17] = "Continue to Demo";

  TLabel                 Label5;
  char Label5_Caption[17] = "mikroElektronika";

  TButton                *Screen1_Buttons[1];
  TLabel                 *Screen1_Labels[5];




static void InitializeObjects() {
  Screen1.Color                     = 0x0000;
  Screen1.Width                     = 320;
  Screen1.Height                    = 240;
  Screen1.ButtonsCount              = 0;
  Screen1.LabelsCount               = 0;
  Screen1.ImagesCount               = 5;
  Screen1.Images                    = Screen0_Images;
  Screen1.CirclesCount              = 1;
  Screen1.Circles                   = Screen0_Circles;
  Screen1.LinesCount                = 3;
  Screen1.Lines                     = Screen0_Lines;
  Screen1.ObjectsCount              = 9;
  Screen0_Lines[0]                  = &Line1;
  Screen0_Lines[1]                  = &Line2;
  Screen0_Circles[0]                = &Circle1;
  Screen0_Images[0]                 = &Image3;
  Screen0_Images[1]                 = &Image1;
  Screen0_Images[2]                 = &Image2;
  Screen0_Images[3]                 = &Image4;
  Screen0_Lines[2]                  = &Line3;
  Screen0_Images[4]                 = &Image5;

  Screen2.Color                     = 0x0000;
  Screen2.Width                     = 320;
  Screen2.Height                    = 240;
  Screen2.ButtonsCount              = 1;
  Screen2.Buttons                   = Screen1_Buttons;
  Screen2.LabelsCount               = 5;
  Screen2.Labels                    = Screen1_Labels;
  Screen2.ImagesCount               = 0;
  Screen2.CirclesCount              = 0;
  Screen2.LinesCount                = 0;
  Screen2.ObjectsCount              = 6;
  Screen1_Labels[0]                 = &Label1;
  Screen1_Labels[1]                 = &Label2;
  Screen1_Labels[2]                 = &Label3;
  Screen1_Labels[3]                 = &Label4;
  Screen1_Buttons[0]                = &Button1;
  Screen1_Labels[4]                 = &Label5;


  Line1.OwnerScreen    = &Screen1;
  Line1.Order          = 0;
  Line1.First_Point_X  = 0;
  Line1.First_Point_Y  = 70;
  Line1.Second_Point_X = 243;
  Line1.Second_Point_Y = 70;
  Line1.Pen_Width      = 7;
  Line1.Visible        = 1;
  Line1.Color          = 0xC618;

  Line2.OwnerScreen    = &Screen1;
  Line2.Order          = 1;
  Line2.First_Point_X  = 78;
  Line2.First_Point_Y  = 150;
  Line2.Second_Point_X = 321;
  Line2.Second_Point_Y = 150;
  Line2.Pen_Width      = 7;
  Line2.Visible        = 1;
  Line2.Color          = 0xC618;

  Circle1.OwnerScreen     = &Screen1;
  Circle1.Order           = 2;
  Circle1.Left            = 12;
  Circle1.Top             = 18;
  Circle1.Radius          = 15;
  Circle1.Pen_Width       = 3;
  Circle1.Pen_Color       = 0xF800;
  Circle1.Visible         = 1;
  Circle1.Active          = 1;
  Circle1.Transparent     = 1;
  Circle1.Gradient        = 0;
  Circle1.Gradient_Orientation    = 0;
  Circle1.Gradient_Start_Color    = 0xFFFF;
  Circle1.Gradient_End_Color      = 0x8410;
  Circle1.Color           = 0x8410;
  Circle1.PressColEnabled = 0;
  Circle1.Press_Color     = 0x8410;
  Circle1.OnUpPtr        = 0;
  Circle1.OnDownPtr      = 0;
  Circle1.OnClickPtr     = 0;
  Circle1.OnPressPtr     = 0;

  Image3.OwnerScreen    = &Screen1;
  Image3.Order          = 3;
  Image3.Left           = 231;
  Image3.Top            = 155;
  Image3.Width          = 30;
  Image3.Height         = 30;
  Image3.Picture_Type   = 1;
  Image3.Picture_Name   = Image3_eve_arrow_up;
  Image3.Visible        = 1;
  Image3.Active         = 1;
  Image3.OnUpPtr        = 0;
  Image3.OnDownPtr      = 0;
  Image3.OnClickPtr     = 0;
  Image3.OnPressPtr     = Image3Press;

  Image1.OwnerScreen    = &Screen1;
  Image1.Order          = 4;
  Image1.Left           = 201;
  Image1.Top            = 183;
  Image1.Width          = 30;
  Image1.Height         = 30;
  Image1.Picture_Type   = 1;
  Image1.Picture_Name   = Image1_eve_arrow_left;
  Image1.Visible        = 1;
  Image1.Active         = 1;
  Image1.OnUpPtr        = 0;
  Image1.OnDownPtr      = 0;
  Image1.OnClickPtr     = 0;
  Image1.OnPressPtr     = Image1Press;

  Image2.OwnerScreen    = &Screen1;
  Image2.Order          = 5;
  Image2.Left           = 264;
  Image2.Top            = 184;
  Image2.Width          = 30;
  Image2.Height         = 30;
  Image2.Picture_Type   = 1;
  Image2.Picture_Name   = Image2_eve_arrow_right;
  Image2.Visible        = 1;
  Image2.Active         = 1;
  Image2.OnUpPtr        = 0;
  Image2.OnDownPtr      = 0;
  Image2.OnClickPtr     = 0;
  Image2.OnPressPtr     = Image2Press;

  Image4.OwnerScreen    = &Screen1;
  Image4.Order          = 6;
  Image4.Left           = 232;
  Image4.Top            = 209;
  Image4.Width          = 30;
  Image4.Height         = 30;
  Image4.Picture_Type   = 1;
  Image4.Picture_Name   = Image4_eve_arrow_down;
  Image4.Visible        = 1;
  Image4.Active         = 1;
  Image4.OnUpPtr        = 0;
  Image4.OnDownPtr      = 0;
  Image4.OnClickPtr     = 0;
  Image4.OnPressPtr     = Image4Press;

  Line3.OwnerScreen    = &Screen1;
  Line3.Order          = 7;
  Line3.First_Point_X  = 195;
  Line3.First_Point_Y  = 152;
  Line3.Second_Point_X = 195;
  Line3.Second_Point_Y = 239;
  Line3.Pen_Width      = 3;
  Line3.Visible        = 1;
  Line3.Color          = 0xC618;

  Image5.OwnerScreen    = &Screen1;
  Image5.Order          = 8;
  Image5.Left           = 292;
  Image5.Top            = 155;
  Image5.Width          = 25;
  Image5.Height         = 25;
  Image5.Picture_Type   = 0;
  Image5.Picture_Ratio  = 1;
  Image5.Picture_Name   = Image5_reload;
  Image5.Visible        = 1;
  Image5.Active         = 1;
  Image5.OnUpPtr        = 0;
  Image5.OnDownPtr      = 0;
  Image5.OnClickPtr     = Image5Click;
  Image5.OnPressPtr     = 0;

  Label1.OwnerScreen    = &Screen2;
  Label1.Order          = 0;
  Label1.Left           = 48;
  Label1.Top            = 55;
  Label1.Width          = 230;
  Label1.Height         = 24;
  Label1.Visible        = 1;
  Label1.Active         = 1;
  Label1.Caption        = Label1_Caption;
  Label1.FontName       = HandelGothic_BT23x25;
  Label1.Font_Pos_Ver   = 53;
  Label1.Font_Color     = 0x001F;
  Label1.OnUpPtr        = 0;
  Label1.OnDownPtr      = 0;
  Label1.OnClickPtr     = 0;
  Label1.OnPressPtr     = 0;

  Label2.OwnerScreen    = &Screen2;
  Label2.Order          = 1;
  Label2.Left           = 13;
  Label2.Top            = 92;
  Label2.Width          = 290;
  Label2.Height         = 12;
  Label2.Visible        = 1;
  Label2.Active         = 1;
  Label2.Caption        = Label2_Caption;
  Label2.FontName       = Tahoma11x13;
  Label2.Font_Pos_Ver   = 90;
  Label2.Font_Color     = 0x07FF;
  Label2.OnUpPtr        = 0;
  Label2.OnDownPtr      = 0;
  Label2.OnClickPtr     = 0;
  Label2.OnPressPtr     = 0;

  Label3.OwnerScreen    = &Screen2;
  Label3.Order          = 2;
  Label3.Left           = 14;
  Label3.Top            = 112;
  Label3.Width          = 286;
  Label3.Height         = 12;
  Label3.Visible        = 1;
  Label3.Active         = 1;
  Label3.Caption        = Label3_Caption;
  Label3.FontName       = Tahoma11x13;
  Label3.Font_Pos_Ver   = 110;
  Label3.Font_Color     = 0x07FF;
  Label3.OnUpPtr        = 0;
  Label3.OnDownPtr      = 0;
  Label3.OnClickPtr     = 0;
  Label3.OnPressPtr     = 0;

  Label4.OwnerScreen    = &Screen2;
  Label4.Order          = 3;
  Label4.Left           = 15;
  Label4.Top            = 131;
  Label4.Width          = 292;
  Label4.Height         = 12;
  Label4.Visible        = 1;
  Label4.Active         = 1;
  Label4.Caption        = Label4_Caption;
  Label4.FontName       = Tahoma11x13;
  Label4.Font_Pos_Ver   = 129;
  Label4.Font_Color     = 0x07FF;
  Label4.OnUpPtr        = 0;
  Label4.OnDownPtr      = 0;
  Label4.OnClickPtr     = 0;
  Label4.OnPressPtr     = 0;

  Button1.OwnerScreen     = &Screen2;
  Button1.Order           = 4;
  Button1.Left            = 186;
  Button1.Top             = 190;
  Button1.Width           = 106;
  Button1.Height          = 33;
  Button1.Pen_Width       = 1;
  Button1.Pen_Color       = 0x0000;
  Button1.Visible         = 1;
  Button1.Active          = 1;
  Button1.Transparent     = 1;
  Button1.Caption         = Button1_Caption;
  Button1.FontName        = Tahoma11x13;
  Button1.Font_Color      = 0x0000;
  Button1.Gradient        = 1;
  Button1.Gradient_Orientation    = 0;
  Button1.Gradient_Start_Color    = 0xFFFF;
  Button1.Gradient_End_Color      = 0x8410;
  Button1.Color           = 0xC618;
  Button1.PressColEnabled = 1;
  Button1.Press_Color     = 0xC618;
  Button1.OnUpPtr        = 0;
  Button1.OnDownPtr      = 0;
  Button1.OnClickPtr     = Button1Click;
  Button1.OnPressPtr     = 0;

  Label5.OwnerScreen    = &Screen2;
  Label5.Order          = 5;
  Label5.Left           = 71;
  Label5.Top            = 22;
  Label5.Width          = 198;
  Label5.Height         = 27;
  Label5.Visible        = 1;
  Label5.Active         = 1;
  Label5.Caption        = Label5_Caption;
  Label5.FontName       = Impact21x29;
  Label5.Font_Pos_Ver   = 20;
  Label5.Font_Color     = 0xF800;
  Label5.OnUpPtr        = 0;
  Label5.OnDownPtr      = 0;
  Label5.OnClickPtr     = 0;
  Label5.OnPressPtr     = 0;
}

static char IsInsideObject (unsigned int X, unsigned int Y, unsigned int Left, unsigned int Top, unsigned int Width, unsigned int Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetButton(index)              CurrentScreen->Buttons[index]
#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetImage(index)               CurrentScreen->Images[index]
#define GetCircle(index)              CurrentScreen->Circles[index]
#define GetLine(index)                CurrentScreen->Lines[index]


void DrawButton(TButton *Abutton) {
  if (Abutton->Visible == 1) {
    TFT_Set_Brush(Abutton->Transparent, Abutton->Color, Abutton->Gradient, Abutton->Gradient_Orientation, Abutton->Gradient_Start_Color, Abutton->Gradient_End_Color);
    TFT_Set_Pen(Abutton->Pen_Color, Abutton->Pen_Width);
    TFT_Rectangle(Abutton->Left, Abutton->Top, Abutton->Left + Abutton->Width - 1, Abutton->Top + Abutton->Height - 1);
    TFT_Set_Font(Abutton->FontName, Abutton->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text_Return_Pos(Abutton->Caption, Abutton->Left, Abutton->Top);
    TFT_Write_Text(Abutton->Caption, (Abutton->Left + ((Abutton->Width - caption_length) / 2)), (Abutton->Top + ((Abutton->Height - caption_height) / 2)));
  }
}

static void DrawButtons() {
  int i;
  TButton *local_button;

  for ( i = 0 ; i < CurrentScreen->ButtonsCount ; i++ ) {
    local_button = GetButton(i);
    DrawButton(local_button);
  }
}


void DrawLabel(TLabel *Alabel) {
  if (Alabel->Visible == 1) {
    TFT_Set_Font(Alabel->FontName, Alabel->Font_Color, FO_HORIZONTAL);
    TFT_Write_Text(Alabel->Caption, Alabel->Left, Alabel->Top);
  }
}

static void DrawLabels() {
  int i;
  TLabel *local_label;

  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    DrawLabel(local_label);
  }
}


void DrawImage(TImage *Aimage) {
  if (Aimage->Visible == 1) {
    if (Aimage->Picture_Type == 0) {
      TFT_Image(Aimage->Left, Aimage->Top, Aimage->Picture_Name, Aimage->Picture_Ratio);
    }
    if (Aimage->Picture_Type == 1) {
      TFT_Image_Jpeg(Aimage->Left, Aimage->Top, Aimage->Picture_Name);
    }
  }
}

static void DrawImages() {
  int i;
  TImage *local_image;

  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    DrawImage(local_image);
  }
}


void DrawCircle(TCircle *Acircle) {
  if (Acircle->Visible == 1) {
    TFT_Set_Brush(Acircle->Transparent, Acircle->Color, Acircle->Gradient, Acircle->Gradient_Orientation,
                  Acircle->Gradient_Start_Color, Acircle->Gradient_End_Color);
    TFT_Set_Pen(Acircle->Pen_Color, Acircle->Pen_Width);
    TFT_Circle(Acircle->Left + Acircle->Radius,
               Acircle->Top + Acircle->Radius,
               Acircle->Radius);
  }
}

static void DrawCircles() {
  int i;
  TCircle *local_circle;

  for ( i = 0 ; i < CurrentScreen->CirclesCount ; i++ ) {
    local_circle = GetCircle(i);
    DrawCircle(local_circle);
  }
}


void DrawLine(TLine *Aline) {
  if (Aline->Visible == 1) {
    TFT_Set_Pen(Aline->Color, Aline->Pen_Width);
    TFT_Line(Aline->First_Point_X, Aline->First_Point_Y, Aline->Second_Point_X, Aline->Second_Point_Y);
  }
}

static void DrawLines() {
  int i;
  TLine *local_line;

  for ( i = 0 ; i < CurrentScreen->LinesCount ; i++ ) {
    local_line = GetLine(i);
    DrawLine(local_line);
  }
}

void DrawScreen(TScreen *aScreen) {
  unsigned short order;
  unsigned short button_idx;
  TButton *local_button;
  unsigned short label_idx;
  TLabel *local_label;
  unsigned short image_idx;
  TImage *local_image;
  unsigned short circle_idx;
  TCircle *local_circle;
  unsigned short line_idx;
  TLine *local_line;
  order = 0;
  button_idx = 0;
  label_idx = 0;
  image_idx = 0;
  circle_idx = 0;
  line_idx = 0;

  if ((display_width != aScreen->Width) || (display_height != aScreen->Height)) {
    TFT_BLED_Direction = 0;
    TFT_BLED           = 0;
    TFT_Init(aScreen->Width, aScreen->Height);
    TP_TFT_Init(aScreen->Width, aScreen->Height, 13, 12);                                  // Initialize touch panel
    TP_TFT_Set_ADC_Threshold(900);                              // Set touch panel ADC threshold
    TFT_Fill_Screen(aScreen->Color);
    display_width = aScreen->Width;
    display_height = aScreen->Height;
    TFT_BLED           = 1;
  }
  else
    TFT_Fill_Screen(aScreen->Color);

  CurrentScreen = aScreen;


  while (order < CurrentScreen->ObjectsCount) {
    if (button_idx < CurrentScreen->ButtonsCount) {
      local_button = GetButton(button_idx);
      if (order == local_button->Order) {
        button_idx++;
        order++;
        DrawButton(local_button);
      }
    }

    if (label_idx < CurrentScreen->LabelsCount) {
      local_label = GetLabel(label_idx);
      if (order == local_label->Order) {
        label_idx++;
        order++;
        DrawLabel(local_label);
      }
    }

    if (circle_idx < CurrentScreen->CirclesCount) {
      local_circle = GetCircle(circle_idx);
      if (order == local_circle->Order) {
        circle_idx++;
        order++;
        DrawCircle(local_circle);
      }
    }

    if (line_idx  < CurrentScreen->LinesCount) {
      local_line = GetLine(line_idx);
      if (order == local_line->Order) {
        line_idx++;
        order++;
        DrawLine(local_line);
      }
    }

    if (image_idx  < CurrentScreen->ImagesCount) {
      local_image = GetImage(image_idx);
      if (order == local_image->Order) {
        image_idx++;
        order++;
        DrawImage(local_image);
      }
    }

  }
}

static void Process_TP_Press(unsigned int X, unsigned int Y) {
  int i;
  TButton *local_button;
  TButton *exec_button;
  short button_order;
  TLabel *local_label;
  TLabel *exec_label;
  short label_order;
  TImage *local_image;
  TImage *exec_image;
  short image_order;
  TCircle *local_circle;
  TCircle *exec_circle;
  short circle_order;

  button_order        = -1;
  label_order         = -1;
  image_order         = -1;
  circle_order        = -1;

  // Buttons
  for ( i = 0 ; i < CurrentScreen->ButtonsCount ; i++ ) {
    local_button = GetButton(i);
    if (local_button->Active == 1) {
      if (IsInsideObject(X, Y, local_button->Left, local_button->Top,
                         local_button->Width, local_button->Height) == 1) {
        button_order = local_button->Order;
        exec_button = local_button;
      }
    }
  }

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  // Images
  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  // Circles
  for ( i = 0 ; i < CurrentScreen->CirclesCount ; i++ ) {
    local_circle = GetCircle(i);
    if (local_circle->Active == 1) {
      if (IsInsideObject(X, Y, local_circle->Left, local_circle->Top,
                        (local_circle->Radius * 2), (local_circle->Radius * 2)) == 1) {
        circle_order = local_circle->Order;
        exec_circle = local_circle;
      }
    }
  }

  i = -1;
  if (button_order > i)
    i = button_order;
  if (label_order > i)
    i = label_order;
  if (image_order > i)
    i = image_order;
  if (circle_order > i)
    i = circle_order;

  if (i != -1) {
    if (i == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->OnPressPtr != 0) {
          exec_button->OnPressPtr();
          return;
        }
      }
    }

    if (i == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnPressPtr != 0) {
          exec_label->OnPressPtr();
          return;
        }
      }
    }

    if (i == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnPressPtr != 0) {
          exec_image->OnPressPtr();
          return;
        }
      }
    }

    if (i == circle_order) {
      if (exec_circle->Active == 1) {
        if (exec_circle->OnPressPtr != 0) {
          exec_circle->OnPressPtr();
          return;
        }
      }
    }

  }
}

static void Process_TP_Up(unsigned int X, unsigned int Y) {
  int i;
  TButton *local_button;
  TButton *exec_button;
  short button_order;
  TLabel *local_label;
  TLabel *exec_label;
  short label_order;
  TImage *local_image;
  TImage *exec_image;
  short image_order;
  TCircle *local_circle;
  TCircle *exec_circle;
  short circle_order;

  switch (PressedObjectType) {
    // Button
    case 0: {
      if (PressedObject != 0) {
        exec_button = (TButton*)PressedObject;
        if (exec_button->PressColEnabled == 1) {
          TFT_Set_Brush(exec_button->Transparent, exec_button->Color, exec_button->Gradient, exec_button->Gradient_Orientation, exec_button->Gradient_Start_Color, exec_button->Gradient_End_Color);
          TFT_Set_Pen(exec_button->Pen_Color, exec_button->Pen_Width);
          TFT_Rectangle(exec_button->Left, exec_button->Top, exec_button->Left + exec_button->Width - 1, exec_button->Top + exec_button->Height - 1);
          TFT_Set_Font(exec_button->FontName, exec_button->Font_Color, FO_HORIZONTAL);
          TFT_Write_Text_Return_Pos(exec_button->Caption, exec_button->Left, exec_button->Top);
          TFT_Write_Text(exec_button->Caption, (exec_button->Left + ((exec_button->Width - caption_length) / 2)), (exec_button->Top + ((exec_button->Height - caption_height) / 2)));
        }
        break;
      }
      break;
    }
    // Circle
    case 4: {
      if (PressedObject != 0) {
        exec_circle = (TCircle*)PressedObject;
        if (exec_circle->PressColEnabled == 1) {
          TFT_Set_Brush(exec_circle->Transparent, exec_circle->Color, exec_circle->Gradient, exec_circle->Gradient_Orientation,
                        exec_circle->Gradient_Start_Color, exec_circle->Gradient_End_Color);
          TFT_Set_Pen(exec_circle->Pen_Color, exec_circle->Pen_Width);
          TFT_Circle(exec_circle->Left + exec_circle->Radius,
                     exec_circle->Top + exec_circle->Radius,
                     exec_circle->Radius);
        }
        break;
      }
      break;
    }
  }

  button_order        = -1;
  label_order         = -1;
  image_order         = -1;
  circle_order        = -1;

  // Buttons
  for ( i = 0 ; i < CurrentScreen->ButtonsCount ; i++ ) {
    local_button = GetButton(i);
    if (local_button->Active == 1) {
      if (IsInsideObject(X, Y, local_button->Left, local_button->Top,
                         local_button->Width, local_button->Height) == 1) {
        button_order = local_button->Order;
        exec_button = local_button;
      }
    }
  }

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  // Images
  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  // Circles
  for ( i = 0 ; i < CurrentScreen->CirclesCount ; i++ ) {
    local_circle = GetCircle(i);
    if (local_circle->Active == 1) {
      if (IsInsideObject(X, Y, local_circle->Left, local_circle->Top,
                        (local_circle->Radius * 2), (local_circle->Radius * 2)) == 1) {
        circle_order = local_circle->Order;
        exec_circle = local_circle;
      }
    }
  }

  i = -1;
  if (button_order > i)
    i = button_order;
  if (label_order > i)
    i = label_order;
  if (image_order > i)
    i = image_order;
  if (circle_order > i)
    i = circle_order;

  if (i != -1) {
  // Buttons
    if (i == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->OnUpPtr != 0)
          exec_button->OnUpPtr();
        if (PressedObject == (void *)exec_button)
          if (exec_button->OnClickPtr != 0)
            exec_button->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Labels
    if (i == label_order) {
      if (exec_label->Active == 1) {
        if (exec_label->OnUpPtr != 0)
          exec_label->OnUpPtr();
        if (PressedObject == (void *)exec_label)
          if (exec_label->OnClickPtr != 0)
            exec_label->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Images
    if (i == image_order) {
      if (exec_image->Active == 1) {
        if (exec_image->OnUpPtr != 0)
          exec_image->OnUpPtr();
        if (PressedObject == (void *)exec_image)
          if (exec_image->OnClickPtr != 0)
            exec_image->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  // Circles
    if (i == circle_order) {
      if (exec_circle->Active == 1) {
        if (exec_circle->OnUpPtr != 0)
          exec_circle->OnUpPtr();
        if (PressedObject == (void *)exec_circle)
          if (exec_circle->OnClickPtr != 0)
            exec_circle->OnClickPtr();
        PressedObject = 0;
        PressedObjectType = -1;
        return;
      }
    }

  }
  PressedObject = 0;
  PressedObjectType = -1;
}

static void Process_TP_Down(unsigned int X, unsigned int Y) {
  int i;
  TButton *local_button;
  TButton *exec_button;
  short button_order;
  TLabel *local_label;
  TLabel *exec_label;
  short label_order;
  TImage *local_image;
  TImage *exec_image;
  short image_order;
  TCircle *local_circle;
  TCircle *exec_circle;
  short circle_order;

  button_order        = -1;
  label_order         = -1;
  image_order         = -1;
  circle_order        = -1;

  // Buttons
  for ( i = 0 ; i < CurrentScreen->ButtonsCount ; i++ ) {
    local_button = GetButton(i);
    if (local_button->Active == 1) {
      if (IsInsideObject(X, Y, local_button->Left, local_button->Top,
                         local_button->Width, local_button->Height) == 1) {
        button_order = local_button->Order;
        exec_button = local_button;
      }
    }
  }

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Active == 1) {
      if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                         local_label->Width, local_label->Height) == 1) {
        label_order = local_label->Order;
        exec_label = local_label;
      }
    }
  }

  // Images
  for ( i = 0 ; i < CurrentScreen->ImagesCount ; i++ ) {
    local_image = GetImage(i);
    if (local_image->Active == 1) {
      if (IsInsideObject(X, Y, local_image->Left, local_image->Top,
                         local_image->Width, local_image->Height) == 1) {
        image_order = local_image->Order;
        exec_image = local_image;
      }
    }
  }

  // Circles
  for ( i = 0 ; i < CurrentScreen->CirclesCount ; i++ ) {
    local_circle = GetCircle(i);
    if (local_circle->Active == 1) {
      if (IsInsideObject(X, Y, local_circle->Left, local_circle->Top,
                        (local_circle->Radius * 2), (local_circle->Radius * 2)) == 1) {
        circle_order = local_circle->Order;
        exec_circle = local_circle;
      }
    }
  }

  i = -1;
  if (button_order > i)
    i = button_order;
  if (label_order > i)
    i = label_order;
  if (image_order > i)
    i = image_order;
  if (circle_order > i)
    i = circle_order;
  if (i != -1) {
    if (i == button_order) {
      if (exec_button->Active == 1) {
        if (exec_button->PressColEnabled == 1) {
          TFT_Set_Brush(exec_button->Transparent, exec_button->Press_Color, exec_button->Gradient, exec_button->Gradient_Orientation, exec_button->Gradient_End_Color, exec_button->Gradient_Start_Color);
          TFT_Set_Pen(exec_button->Pen_Color, exec_button->Pen_Width);
          TFT_Rectangle(exec_button->Left, exec_button->Top, exec_button->Left + exec_button->Width - 1, exec_button->Top + exec_button->Height - 1);
          TFT_Set_Font(exec_button->FontName, exec_button->Font_Color, FO_HORIZONTAL);
          TFT_Write_Text_Return_Pos(exec_button->Caption, exec_button->Left, exec_button->Top);
          TFT_Write_Text(exec_button->Caption, (exec_button->Left + ((exec_button->Width - caption_length) / 2)), (exec_button->Top + ((exec_button->Height - caption_height) / 2)));
        }
        PressedObject = (void *)exec_button;
        PressedObjectType = 0;
        if (exec_button->OnDownPtr != 0) {
          exec_button->OnDownPtr();
          return;
        }
      }
    }

    if (i == label_order) {
      if (exec_label->Active == 1) {
        PressedObject = (void *)exec_label;
        PressedObjectType = 2;
        if (exec_label->OnDownPtr != 0) {
          exec_label->OnDownPtr();
          return;
        }
      }
    }

    if (i == image_order) {
      if (exec_image->Active == 1) {
        PressedObject = (void *)exec_image;
        PressedObjectType = 3;
        if (exec_image->OnDownPtr != 0) {
          exec_image->OnDownPtr();
          return;
        }
      }
    }

    if (i == circle_order) {
      if (exec_circle->Active == 1) {
        if (exec_circle->PressColEnabled == 1) {
          TFT_Set_Brush(exec_circle->Transparent, exec_circle->Press_Color, exec_circle->Gradient, exec_circle->Gradient_Orientation,
                        exec_circle->Gradient_End_Color, exec_circle->Gradient_Start_Color);
          TFT_Set_Pen(exec_circle->Pen_Color, exec_circle->Pen_Width);
          TFT_Circle(exec_circle->Left + exec_circle->Radius,
                     exec_circle->Top + exec_circle->Radius,
                     exec_circle->Radius);
        }
        PressedObject = (void *)exec_circle;
        PressedObjectType = 4;
        if (exec_circle->OnDownPtr != 0) {
          exec_circle->OnDownPtr();
          return;
        }
      }
    }

  }
}

void Check_TP() {
  if (TP_TFT_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to Display dimensions space
    if (TP_TFT_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Start_TP() {
  InitializeTouchPanel();

  // You can get calibration constants using touch panel calibration example
  TP_TFT_Set_Calibration_Consts(149, 776, 68, 765);    // Set calibration constants

  InitializeObjects();
  display_width = Screen2.Width;
  display_height = Screen2.Height;
  DrawScreen(&Screen2);
}
